function varargout = ISIGeneration (NbRuns, NbEvents, MeanISIs, MinISIs, ...
			                        slice_time, slice_per_tr, begin_end)

% ===================================================================
% Generation d'Intervalles Inter-Stimuli  distribution exponentielle
%
% Usage : ISIGeneration 
%
%   NbRuns       : Nombre de runs  gnrer
%   NbEvents     : Nombre d'vnements par run
%   MeanISIs     : Intervalle Inter-stimuli moyen (en s)
%   MinISIs      : Intervalle Inter-stimuli minimum (en s)
%   slice_time	 : Dure d'une coupe en secondes
%   slice_per_tr : Nombre de coupes par TR
%   begin_end    : Debut et Fin de Baseline en secondes
%
% Remerciement  Matthew Brett qui est  l'origine de cette routine
% ===================================================================

% Un gnre un nombre "global" d'ISI pour optimiser la distribution
Nb_desired_events = NbRuns * NbEvents  ;

mean_isi_sl = MeanISIs / slice_time  ;  % ISI moyen exprim en slices
min_isi_sl  = MinISIs / slice_time   ;  % ISI minimum exprim en slices

mean_isi_tr = mean_isi_sl / slice_per_tr;
min_isi_tr = min_isi_sl / slice_per_tr;


%% On se garde une marge correspondant a la moitie des scans finaux hors les 2 derniers
% dummy scans.
nslices_max = NbEvents * MeanISIs + ( begin_end(2) - 2 * slice_per_tr * slice_time) / 2 ;
nslices_max = NbRuns * nslices_max / slice_time;

Max = 99999999;
while (Max > nslices_max / NbRuns)
st = 99999999 * ones(1,Nb_desired_events);

while max(st) > nslices_max
   % Gnration des temps cumuls  partir d'une distribution exponentielle randomise
   st = cumsum((-log(rand(1,Nb_desired_events))*(mean_isi_sl-min_isi_sl))+min_isi_sl);
end

st = round(st') ;

% Gnration des ISIs
Inter = zeros(Nb_desired_events,1) ;
Stim  = Inter ;
Inter(1,1) = st(1,1) ;

for i=2:Nb_desired_events
   Inter(i,1) = st(i,1)-st(i-1,1) ;
   % Inter(i,1) = round (st(i,1)-st(i-1,1)) * slice_per_tr ;
end
figure, hist(Inter);

% On classe les ISI et on les tire dans l'ordre dcroissant
% pour les dispatcher (1 par 1) dans chaque run (c) JLA
InterSort = sort(Inter) ;
ISIS = zeros(NbRuns,NbEvents) ;
RandISIS = zeros(NbRuns,NbEvents) ;
for i=Nb_desired_events:-1*NbRuns:NbRuns
    for j=1:NbRuns
        ISIS(j,floor(i/NbRuns)) = InterSort(i-j+1);
    end
end

% Calcul des dures de chacun des runs dans ce premier essai
FirstRunLengths = sum(ISIS') ;

% Recherche du Run le plus long
Max = FirstRunLengths(1) ;
end     %% fin du while sur la duree du run le plus long : Max < 99999999

Maxi = 1 ;
for i=2:NbRuns
    if (FirstRunLengths(i)>Max)
        Max = FirstRunLengths(i) ;
        Maxi = i ;
    end
end

% On distribue la diffrence sur les derniers ISI de chaque Run
for j=1:NbRuns
    if (j~=Maxi)
        Rab = Max - FirstRunLengths(j) ;
        k = 0;
        while Rab > 0
            if (Rab == 1)
                Ajout = 1 ;
            else
                Ajout = floor(Rab/2) ;
            end
            ISIS(j,NbEvents-k) = ISIS(j,NbEvents-k) + Ajout ;
            Rab = Rab - Ajout ;
            k = k + 1 ;
        end
    end
end

% Maintenant, il faut tirer au sort les positions des ISIs gnrs
for j=1:NbRuns
    Perm = randperm(NbEvents) ; % Permutation alatoire des ISIS
    for (i=1:NbEvents)
        RandISIS(j,i) = ISIS(j,Perm(i)) ;
    end
end
 
varargout = {RandISIS};